@@ -72,16 +72,17 @@ def adduser(username, user_id, groups, create_home):
         with open(ETC_GROUP, 'w') as etc_group_file:
             etc_group_file.write(new_etc_group_file)
     # 5. Create user directory at /home and copy the template from /etc/skel/.*
+    home_dir = HOME + "/" + username
     if create_home: # This step is optional
-        home_dir = HOME + "/" + username
         shutil.copytree(HOME_TEMPLATE, home_dir) # Create home directory from the skeleton files
-        os.chown(home_dir, user_id, user_id) # Set as owner the user and the user group to the files on the user dir
-        for dirpath, dirnames, filenames in os.walk(home_dir):
-            for dname in dirnames:
-                os.chown(os.path.join(dirpath, dname), user_id, user_id)
-            for fname in filenames:
-                os.chown(os.path.join(dirpath, fname), user_id, user_id)
-        os.chmod(home_dir, 0o700) # Only allows owner to list files
+    # 6. Set rights of existing user directory
+    os.chown(home_dir, user_id, user_id) # Set as owner the user and the user group to the files on the user dir
+    for dirpath, dirnames, filenames in os.walk(home_dir):
+        for dname in dirnames:
+            os.chown(os.path.join(dirpath, dname), user_id, user_id)
+        for fname in filenames:
+            os.chown(os.path.join(dirpath, fname), user_id, user_id)
+    os.chmod(home_dir, 0o700) # Only allows owner to list files
 ##
 
 class OpenbisAuthenticator(Authenticator):
@@ -90,6 +91,12 @@ class OpenbisAuthenticator(Authenticator):
         help='URL of openBIS server to contact'
     )
 
+    use_kerberos = Bool(
+        config=False,
+        default_value=False,
+        help='Indicates if Kerberos is going to be used to copy username data to environment variables'
+    )
+
     kerberos_domain = Unicode(
         config=True,
         help='The domain to authenticate against to obtain a kerberso ticket'
@@ -159,7 +166,8 @@ class OpenbisAuthenticator(Authenticator):
                 return None
         else:
             if session.find('-') > -1:
-                username = session[0:session.index('-')]
+                lastIndexOfMinus = len(session) - "".join(reversed(session)).index("-") - 1
+                username = session[0:lastIndexOfMinus]
                 password = ''
             else:
                 self.log.warn('Invalid session')
@@ -167,15 +175,13 @@ class OpenbisAuthenticator(Authenticator):
 
         openbis = Openbis(self.server_url, verify_certificates=self.verify_certificates)
         try:
-            # authenticate against openBIS and store the token (if possible)
+            # authenticate against openBIS and store the token
             if session is None or session.strip() == '':
-                openbis.login(username, password, True)
+                openbis.login(username, password)
             else:
-                openbis.token = session
+                openbis.set_token(session)
                 if openbis.is_session_active():
                     self.log.warn('Valid session')
-                    if os.environ.get('OPENBIS_URL') == openbis.url:
-                        os.environ['OPENBIS_TOKEN'] = openbis.token
                 else:
                     self.log.warn('Invalid session')
                     return None
@@ -185,6 +191,8 @@ class OpenbisAuthenticator(Authenticator):
                     user = pwd.getpwnam(username)
                 except KeyError:
                     self.create_user(username)
+            # Save token after creating the user
+            openbis.save_token_on_behalf(os_home = HOME + "/" + username)
 
             # instead of just returning the username, we return a dict
             # containing the auth_state as well
@@ -199,10 +207,10 @@ class OpenbisAuthenticator(Authenticator):
                 "auth_state": {
                     "token": openbis.token,
                     "url": openbis.url,
-                    "kerberos_username": kerberos_username,
                     "username": username,
-                    "kerberos_password": password,
-
+                    "use_kerberos": self.use_kerberos,
+                    "kerberos_username": kerberos_username,
+                    "kerberos_password": password
                 }
             }
         except ValueError as err:
@@ -217,13 +225,14 @@ class OpenbisAuthenticator(Authenticator):
         if not auth_state:
             # auth_state not enabled
             return
-
-        # Write the openBIS token to the users' environment variables
-        spawner.environment['OPENBIS_URL']       = auth_state['url']
-        spawner.environment['OPENBIS_TOKEN']     = auth_state['token']
-        spawner.environment['KERBEROS_USERNAME'] = auth_state['kerberos_username']
-        spawner.environment['KERBEROS_PASSWORD'] = auth_state['kerberos_password']
-        spawner.environment['USERNAME']          = auth_state['username']
+        # Write the Authenticator openBIS URL to the user environment variables
+        spawner.environment['OPENBIS_URL'] = auth_state['url']
+        # Save the username and password to get a kerberos ticket on the future if kerberos is configured
+        if auth_state['use_kerberos']:
+            spawner.environment['KERBEROS_USERNAME'] = auth_state['kerberos_username']
+            spawner.environment['KERBEROS_PASSWORD'] = auth_state['kerberos_password']
+        del auth_state['kerberos_username']
+        del auth_state['kerberos_password']
 
 
     def get_new_uid_for_home(self, os_home):
@@ -265,6 +274,8 @@ class KerberosSudoSpawner(SudoSpawner):
             return env
         env['KERBEROS_USERNAME'] = spawner_env['KERBEROS_USERNAME']
         env['KERBEROS_PASSWORD'] = spawner_env['KERBEROS_PASSWORD']
+        del spawner_env['KERBEROS_USERNAME']
+        del spawner_env['KERBEROS_PASSWORD']
         return env
 
     @gen.coroutine
@@ -274,11 +285,14 @@ class KerberosSudoSpawner(SudoSpawner):
         without a valid Kerberos ticket.
         """
 
-        if os.getenv('KERBEROS_USERNAME') and os.getenv('KERBEROS_PASSWORD'):
+        if os.environ['KERBEROS_USERNAME'] and os.environ['KERBEROS_PASSWORD']:
+            kerberos_username = os.environ['KERBEROS_USERNAME']
+            kerberos_password = os.environ['KERBEROS_PASSWORD']
+            del os.environ['KERBEROS_USERNAME']
+            del os.environ['KERBEROS_PASSWORD']
             pamela.authenticate(
-                os.getenv('KERBEROS_USERNAME'),
-                os.getenv('KERBEROS_PASSWORD'),
+                kerberos_username,
+                kerberos_password,
                 resetcred=0x0002
             )
-
         return super().do(action, **kwargs)
